import { DoorLockService } from '../services/door.lock.service';
import {
  Body,
  Controller,
  Post,
  Param,
  HttpStatus,
  Get,
  Delete,
  UseGuards,
  Put,
} from '@nestjs/common';
import { ApiTags, ApiBearerAuth, ApiOperation } from '@nestjs/swagger';
import { AddDoorLockOnlineDto } from '../dtos/add.online-temp.dto';
import { AddDoorLockOfflineTempMultipleTimeDto } from '../dtos/add.offline-temp.dto';
import { JwtAuthGuard } from '@app/common/guards/jwt.auth.guard';
import { UpdateDoorLockOfflineTempDto } from '../dtos/update.offline-temp.dto';
import { EnableDisableStatusEnum } from '@app/common/constants/days.enum';
import { ControllerRoute } from '@app/common/constants/controller-route';

@ApiTags('Door Lock Module')
@Controller({
  version: EnableDisableStatusEnum.ENABLED,
  path: ControllerRoute.DOOR_LOCK.ROUTE,
})
export class DoorLockController {
  constructor(private readonly doorLockService: DoorLockService) {}
  @ApiBearerAuth()
  @UseGuards(JwtAuthGuard)
  @Post('temporary-password/online/:doorLockUuid')
  @ApiOperation({
    summary:
      ControllerRoute.DOOR_LOCK.ACTIONS.ADD_ONLINE_TEMPORARY_PASSWORD_SUMMARY,
    description:
      ControllerRoute.DOOR_LOCK.ACTIONS
        .ADD_ONLINE_TEMPORARY_PASSWORD_DESCRIPTION,
  })
  async addOnlineTemporaryPassword(
    @Body() addDoorLockDto: AddDoorLockOnlineDto,
    @Param('doorLockUuid') doorLockUuid: string,
  ) {
    const temporaryPassword =
      await this.doorLockService.addOnlineTemporaryPassword(
        addDoorLockDto,
        doorLockUuid,
      );

    return {
      statusCode: HttpStatus.CREATED,
      success: true,
      message: 'Online temporary password added successfully',
      data: {
        id: temporaryPassword.id,
      },
    };
  }

  @ApiBearerAuth()
  @UseGuards(JwtAuthGuard)
  @Post('temporary-password/offline/one-time/:doorLockUuid')
  @ApiOperation({
    summary:
      ControllerRoute.DOOR_LOCK.ACTIONS
        .ADD_OFFLINE_ONE_TIME_TEMPORARY_PASSWORD_SUMMARY,
    description:
      ControllerRoute.DOOR_LOCK.ACTIONS
        .ADD_OFFLINE_ONE_TIME_TEMPORARY_PASSWORD_DESCRIPTION,
  })
  async addOfflineOneTimeTemporaryPassword(
    @Param('doorLockUuid') doorLockUuid: string,
  ) {
    const temporaryPassword =
      await this.doorLockService.addOfflineOneTimeTemporaryPassword(
        doorLockUuid,
      );

    return {
      statusCode: HttpStatus.CREATED,
      success: true,
      message: 'Offline one-time temporary password added successfully',
      data: temporaryPassword,
    };
  }

  @ApiBearerAuth()
  @UseGuards(JwtAuthGuard)
  @Post('temporary-password/offline/multiple-time/:doorLockUuid')
  @ApiOperation({
    summary:
      ControllerRoute.DOOR_LOCK.ACTIONS
        .ADD_OFFLINE_MULTIPLE_TIME_TEMPORARY_PASSWORD_SUMMARY,
    description:
      ControllerRoute.DOOR_LOCK.ACTIONS
        .ADD_OFFLINE_MULTIPLE_TIME_TEMPORARY_PASSWORD_DESCRIPTION,
  })
  async addOfflineMultipleTimeTemporaryPassword(
    @Body()
    addDoorLockOfflineTempMultipleTimeDto: AddDoorLockOfflineTempMultipleTimeDto,
    @Param('doorLockUuid') doorLockUuid: string,
  ) {
    const temporaryPassword =
      await this.doorLockService.addOfflineMultipleTimeTemporaryPassword(
        addDoorLockOfflineTempMultipleTimeDto,
        doorLockUuid,
      );

    return {
      statusCode: HttpStatus.CREATED,
      success: true,
      message: 'Offline multiple-time temporary password added successfully',
      data: temporaryPassword,
    };
  }

  @ApiBearerAuth()
  @UseGuards(JwtAuthGuard)
  @Get('temporary-password/online/:doorLockUuid')
  @ApiOperation({
    summary:
      ControllerRoute.DOOR_LOCK.ACTIONS.GET_ONLINE_TEMPORARY_PASSWORDS_SUMMARY,
    description:
      ControllerRoute.DOOR_LOCK.ACTIONS
        .GET_ONLINE_TEMPORARY_PASSWORDS_DESCRIPTION,
  })
  async getOnlineTemporaryPasswords(
    @Param('doorLockUuid') doorLockUuid: string,
  ) {
    return await this.doorLockService.getOnlineTemporaryPasswordsMultiple(
      doorLockUuid,
    );
  }

  @ApiBearerAuth()
  @UseGuards(JwtAuthGuard)
  @Delete('temporary-password/online/:doorLockUuid/:passwordId')
  @ApiOperation({
    summary:
      ControllerRoute.DOOR_LOCK.ACTIONS
        .DELETE_ONLINE_TEMPORARY_PASSWORD_SUMMARY,
    description:
      ControllerRoute.DOOR_LOCK.ACTIONS
        .DELETE_ONLINE_TEMPORARY_PASSWORD_DESCRIPTION,
  })
  async deleteDoorLockPassword(
    @Param('doorLockUuid') doorLockUuid: string,
    @Param('passwordId') passwordId: string,
  ) {
    await this.doorLockService.deleteDoorLockPassword(doorLockUuid, passwordId);
    return {
      statusCode: HttpStatus.OK,
      message: 'Temporary password deleted successfully',
    };
  }

  @ApiBearerAuth()
  @UseGuards(JwtAuthGuard)
  @Get('temporary-password/offline/one-time/:doorLockUuid')
  @ApiOperation({
    summary:
      ControllerRoute.DOOR_LOCK.ACTIONS
        .GET_OFFLINE_ONE_TIME_TEMPORARY_PASSWORDS_SUMMARY,
    description:
      ControllerRoute.DOOR_LOCK.ACTIONS
        .GET_OFFLINE_ONE_TIME_TEMPORARY_PASSWORDS_DESCRIPTION,
  })
  async getOfflineOneTimeTemporaryPasswords(
    @Param('doorLockUuid') doorLockUuid: string,
  ) {
    return await this.doorLockService.getOfflineOneTimeTemporaryPasswords(
      doorLockUuid,
    );
  }

  @ApiBearerAuth()
  @UseGuards(JwtAuthGuard)
  @Get('temporary-password/offline/multiple-time/:doorLockUuid')
  @ApiOperation({
    summary:
      ControllerRoute.DOOR_LOCK.ACTIONS
        .GET_OFFLINE_MULTIPLE_TIME_TEMPORARY_PASSWORDS_SUMMARY,
    description:
      ControllerRoute.DOOR_LOCK.ACTIONS
        .GET_OFFLINE_MULTIPLE_TIME_TEMPORARY_PASSWORDS_DESCRIPTION,
  })
  async getOfflineMultipleTimeTemporaryPasswords(
    @Param('doorLockUuid') doorLockUuid: string,
  ) {
    return await this.doorLockService.getOfflineMultipleTimeTemporaryPasswords(
      doorLockUuid,
    );
  }

  @ApiBearerAuth()
  @UseGuards(JwtAuthGuard)
  @Put('temporary-password/:doorLockUuid/offline/:passwordId')
  @ApiOperation({
    summary:
      ControllerRoute.DOOR_LOCK.ACTIONS
        .UPDATE_OFFLINE_TEMPORARY_PASSWORD_SUMMARY,
    description:
      ControllerRoute.DOOR_LOCK.ACTIONS
        .UPDATE_OFFLINE_TEMPORARY_PASSWORD_DESCRIPTION,
  })
  async updateOfflineTemporaryPassword(
    @Body() updateDoorLockOfflineTempDto: UpdateDoorLockOfflineTempDto,
    @Param('doorLockUuid') doorLockUuid: string,
    @Param('passwordId') passwordId: string,
  ) {
    const temporaryPassword =
      await this.doorLockService.updateOfflineTemporaryPassword(
        updateDoorLockOfflineTempDto,
        doorLockUuid,
        passwordId,
      );

    return {
      statusCode: HttpStatus.CREATED,
      success: true,
      message: 'Offline temporary password updated successfully',
      data: temporaryPassword,
    };
  }

  @ApiBearerAuth()
  @UseGuards(JwtAuthGuard)
  @Post('open/:doorLockUuid')
  @ApiOperation({
    summary: ControllerRoute.DOOR_LOCK.ACTIONS.OPEN_DOOR_LOCK_SUMMARY,
    description: ControllerRoute.DOOR_LOCK.ACTIONS.OPEN_DOOR_LOCK_DESCRIPTION,
  })
  async openDoorLock(@Param('doorLockUuid') doorLockUuid: string) {
    await this.doorLockService.openDoorLock(doorLockUuid);

    return {
      statusCode: HttpStatus.CREATED,
      success: true,
      message: 'Door lock opened successfully',
    };
  }
}
