import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { TuyaContext } from '@tuya/tuya-connector-nodejs';
import { ConfigService } from '@nestjs/config';
import { SpaceRepository } from '@app/common/modules/space/repositories';
import { PermissionType } from '@app/common/constants/permission-type.enum';
import { In } from 'typeorm';
import { ProductType } from '@app/common/constants/product-type.enum';
import { DeviceService } from 'src/device/services';

@Injectable()
export class GroupService {
  private tuya: TuyaContext;
  constructor(
    private readonly configService: ConfigService,
    private readonly deviceService: DeviceService,
    private readonly spaceRepository: SpaceRepository,
  ) {
    const accessKey = this.configService.get<string>('auth-config.ACCESS_KEY');
    const secretKey = this.configService.get<string>('auth-config.SECRET_KEY');
    const tuyaEuUrl = this.configService.get<string>('tuya-config.TUYA_EU_URL');
    this.tuya = new TuyaContext({
      baseUrl: tuyaEuUrl,
      accessKey,
      secretKey,
    });
  }

  async getGroupsBySpaceUuid(spaceUuid: string) {
    try {
      const spaces = await this.spaceRepository.find({
        where: {
          uuid: spaceUuid,
        },
        relations: ['devices', 'devices.productDevice'],
      });

      const groupNames = spaces.flatMap((space) => {
        return space.devices.map((device) => device.productDevice.prodType);
      });

      const uniqueGroupNames = [...new Set(groupNames)];
      const groups = uniqueGroupNames.map((groupName) => ({
        groupName: groupName as ProductType,
      }));
      const allowedProductTypes = [
        ProductType.ONE_1TG,
        ProductType.TWO_2TG,
        ProductType.THREE_3TG,
        ProductType.THREE_G,
        ProductType.TWO_G,
        ProductType.ONE_G,
        ProductType.GD,
        ProductType.WH,
        ProductType.AC,
        ProductType.CUR,
      ];
      return groups.filter((group) =>
        allowedProductTypes.includes(group.groupName),
      );
    } catch (error) {
      throw new HttpException(
        'This space does not have any groups',
        HttpStatus.NOT_FOUND,
      );
    }
  }

  async getSpaceDevicesByGroupName(
    spaceUuid: string,
    groupName: string,
    userUuid: string,
  ) {
    try {
      const spaces = await this.spaceRepository.find({
        where: {
          uuid: spaceUuid,
          devices: {
            productDevice: {
              prodType: groupName,
            },
            permission: {
              userUuid,
              permissionType: {
                type: In([PermissionType.READ, PermissionType.CONTROLLABLE]),
              },
            },
          },
        },
        relations: [
          'devices',
          'devices.productDevice',
          'devices.spaceDevice',
          'devices.permission',
          'devices.permission.permissionType',
        ],
      });

      const devices = await Promise.all(
        spaces.flatMap(async (space) => {
          return await Promise.all(
            space.devices.map(async (device) => {
              const deviceDetails =
                await this.deviceService.getDeviceDetailsByDeviceIdTuya(
                  device.deviceTuyaUuid,
                );
              return {
                haveRoom: !!device.spaceDevice,
                productUuid: device.productDevice.uuid,
                productType: device.productDevice.prodType,
                permissionType: device.permission[0]?.permissionType?.type,
                ...deviceDetails,
                uuid: device.uuid,
              };
            }),
          );
        }),
      );
      if (devices.length === 0)
        throw new HttpException('No devices found', HttpStatus.NOT_FOUND);
      return devices.flat(); // Flatten the array since flatMap was used
    } catch (error) {
      throw new HttpException(
        'This space does not have any devices for the specified group name',
        HttpStatus.NOT_FOUND,
      );
    }
  }
}
