import {
  Injectable,
  NestInterceptor,
  ExecutionContext,
  CallHandler,
  Inject,
} from '@nestjs/common';
import { Observable, throwError } from 'rxjs';
import { map, catchError } from 'rxjs/operators';
import { WINSTON_MODULE_PROVIDER } from 'nest-winston';
import { Logger } from 'winston';
import { v4 as uuidv4 } from 'uuid';

@Injectable()
export class LoggingInterceptor implements NestInterceptor {
  constructor(
    @Inject(WINSTON_MODULE_PROVIDER)
    private readonly logger: Logger,
  ) {}

  intercept(context: ExecutionContext, next: CallHandler): Observable<any> {
    const request = context.switchToHttp().getRequest();
    const res = context.switchToHttp().getResponse();

    const { method, url, body } = request;
    const requestId = uuidv4();
    const now = Date.now();

    res.setHeader('X-Request-Id', requestId);

    return next.handle().pipe(
      map((response) => {
        const duration = Date.now() - now;
        const filteredRequestBody = this.filterSensitiveFields(body);
        const filteredResponse = this.filterSensitiveFields(response);

        this.logger.info('--- Incoming Request ---', {
          method,
          url,
          duration: `${duration}ms`,
          requestBody: filteredRequestBody,
          responseBody: filteredResponse,
        });

        this.logger.info('--- Outgoing Response ---', {
          method,
          url,
          response: filteredResponse,
        });

        return response;
      }),
      catchError((error) => {
        const duration = Date.now() - now;

        this.logger.error('--- Request Error ---', {
          requestId,
          method,
          url,
          message: error.message,
          duration: `${duration}ms`,
          stack: error.stack,
        });
        return throwError(() => error);
      }),
    );
  }

  private filterSensitiveFields(data: any): any {
    const blacklist = ['password', 'refreshToken', 'accessToken', 'otp'];

    if (data && typeof data === 'object' && !Array.isArray(data)) {
      return Object.keys(data).reduce((acc, key) => {
        if (blacklist.includes(key)) {
          acc[key] = '[FILTERED]';
        } else if (typeof data[key] === 'object' && data[key] !== null) {
          acc[key] = this.filterSensitiveFields(data[key]);
        } else {
          acc[key] = data[key];
        }
        return acc;
      }, {});
    } else if (Array.isArray(data)) {
      return data.map((item) => this.filterSensitiveFields(item));
    }

    return data;
  }
}
