import { BaseResponseDto } from '@app/common/dto/base.response.dto';
import { SuccessResponseDto } from '@app/common/dto/success.response.dto';
import {
  TypeORMCustomModel,
  TypeORMCustomModelFindAllQuery,
} from '@app/common/models/typeOrmCustom.model';
import { ProjectEntity } from '@app/common/modules/project/entities';
import {
  SpaceProductAllocationRepository,
  SpaceRepository,
} from '@app/common/modules/space';
import {
  SpaceModelEntity,
  SpaceModelRepository,
} from '@app/common/modules/space-model';
import { SpaceProductAllocationEntity } from '@app/common/modules/space/entities/space-product-allocation.entity';
import { SpaceEntity } from '@app/common/modules/space/entities/space.entity';
import { SubspaceProductAllocationEntity } from '@app/common/modules/space/entities/subspace/subspace-product-allocation.entity';
import { SubspaceEntity } from '@app/common/modules/space/entities/subspace/subspace.entity';
import {
  SubspaceProductAllocationRepository,
  SubspaceRepository,
} from '@app/common/modules/space/repositories/subspace.repository';
import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { CommandBus } from '@nestjs/cqrs';
import { ProjectParam } from 'src/community/dtos';
import { DeviceService } from 'src/device/services';
import { ProjectService } from 'src/project/services';
import { ProcessTagDto } from 'src/tags/dtos';
import { DataSource, In, QueryRunner, SelectQueryBuilder } from 'typeorm';
import {
  PropogateDeleteSpaceModelCommand,
  PropogateUpdateSpaceModelCommand,
  PropogateUpdateSpaceModelProductAllocationCommand,
} from '../commands';
import {
  CreateSpaceModelDto,
  LinkSpacesToModelDto,
  UpdateSpaceModelDto,
} from '../dtos';
import { SpaceModelParam } from '../dtos/space-model-param';
import { ISubspaceModelUpdates, IUpdatedSpaceAllocations } from '../interfaces';
import { SpaceModelProductAllocationService } from './space-model-product-allocation.service';
import { SubSpaceModelService } from './subspace';

@Injectable()
export class SpaceModelService {
  constructor(
    private readonly dataSource: DataSource,
    private readonly spaceModelRepository: SpaceModelRepository,
    private readonly projectService: ProjectService,
    private readonly deviceService: DeviceService,
    private readonly subSpaceModelService: SubSpaceModelService,
    private commandBus: CommandBus,
    private readonly spaceModelProductAllocationService: SpaceModelProductAllocationService,
    private readonly spaceRepository: SpaceRepository,
    private readonly spaceProductAllocationRepository: SpaceProductAllocationRepository,
    private readonly subspaceRepository: SubspaceRepository,
    private readonly subspaceProductAllocationRepository: SubspaceProductAllocationRepository,
  ) {}

  async createSpaceModel(
    createSpaceModelDto: CreateSpaceModelDto,
    params: ProjectParam,
  ): Promise<BaseResponseDto> {
    const { modelName, subspaceModels, tags } = createSpaceModelDto;
    const queryRunner = this.dataSource.createQueryRunner();

    await queryRunner.connect();
    await queryRunner.startTransaction();

    try {
      const project = await this.projectService.findOne(params.projectUuid);
      if (!project) {
        throw new HttpException('Project not found', HttpStatus.NOT_FOUND);
      }

      await this.validateNameUsingQueryRunner(
        modelName,
        params.projectUuid,
        queryRunner,
      );

      const spaceModel = this.spaceModelRepository.create({
        modelName,
        project,
      });

      const savedSpaceModel = await queryRunner.manager.save(spaceModel);

      const subspaceTags =
        this.subSpaceModelService.extractTagsFromSubspaceModels(subspaceModels);

      const allTags = [...tags, ...subspaceTags];
      this.validateUniqueTags(allTags);

      if (subspaceModels?.length) {
        savedSpaceModel.subspaceModels =
          await this.subSpaceModelService.createModels(
            savedSpaceModel,
            subspaceModels,
            queryRunner,
          );
      }

      if (tags?.length) {
        await this.spaceModelProductAllocationService.createProductAllocations(
          params.projectUuid,
          spaceModel,
          tags,
          queryRunner,
        );
      }

      await queryRunner.commitTransaction();

      return new SuccessResponseDto({
        message: `Successfully created new space model with uuid ${savedSpaceModel.uuid}`,
        data: savedSpaceModel,
        statusCode: HttpStatus.CREATED,
      });
    } catch (error) {
      await queryRunner.rollbackTransaction();

      const errorMessage =
        error instanceof HttpException
          ? error.message
          : `An unexpected error occurred: ${error.message}`;
      const statusCode =
        error instanceof HttpException
          ? error.getStatus()
          : HttpStatus.INTERNAL_SERVER_ERROR;

      throw new HttpException(errorMessage, statusCode);
    } finally {
      await queryRunner.release();
    }
  }

  async list(
    param: ProjectParam,
    pageable: Partial<TypeORMCustomModelFindAllQuery>,
  ) {
    await this.validateProject(param.projectUuid);

    try {
      pageable.modelName = 'space-model';
      pageable.where = {
        project: { uuid: param.projectUuid },
        disabled: false,
      };
      pageable.include =
        'subspaceModels.productAllocations,subspaceModelProductAllocations.tag,subspaceModelProductAllocations.product,subspaceModels, productAllocations, productAllocations.tag,productAllocations.product';

      const queryBuilder = this.buildSpaceModelQuery(param.projectUuid);

      const customModel = TypeORMCustomModel(this.spaceModelRepository);
      const { baseResponseDto, paginationResponseDto } =
        await customModel.findAll(
          {
            page: pageable.page || 1,
            size: pageable.size || 10,
            modelName: 'spaceModel',
          },
          queryBuilder,
        );

      const formattedData = this.transformSpaceModelData(baseResponseDto.data);

      return {
        code: 200,
        data: formattedData,
        message: 'Success get list spaceModel',
        ...paginationResponseDto,
      };
    } catch (error) {
      throw new HttpException(
        `Error fetching paginated list: ${error.message}`,
        HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }

  async validateProject(projectUuid: string): Promise<ProjectEntity> {
    return await this.projectService.findOne(projectUuid);
  }

  async update(dto: UpdateSpaceModelDto, param: SpaceModelParam) {
    const queryRunner = this.dataSource.createQueryRunner();
    const project = await this.validateProject(param.projectUuid);
    const spaceModel = await this.validateSpaceModel(
      param.spaceModelUuid,
      param.projectUuid,
    );
    await queryRunner.connect();
    let modifiedSubspaces: ISubspaceModelUpdates;
    let modifiedProductAllocations: IUpdatedSpaceAllocations[];
    try {
      await queryRunner.startTransaction();
      const spaces = await this.fetchModelSpaces(spaceModel);

      const { modelName } = dto;
      if (modelName) {
        await this.validateNameUsingQueryRunner(
          modelName,
          param.projectUuid,
          queryRunner,
        );
        spaceModel.modelName = modelName;
        await queryRunner.manager.save(
          this.spaceModelRepository.target,
          spaceModel,
        );
      }

      if (dto.subspaceModels?.length > 0) {
        modifiedSubspaces =
          await this.subSpaceModelService.modifySubspaceModels(
            dto.subspaceModels,
            spaceModel,
            queryRunner,
            param.projectUuid,
            // dto.tags,
          );
      }

      if (dto.tags?.length) {
        modifiedProductAllocations =
          await this.spaceModelProductAllocationService.updateProductAllocations(
            dto.tags,
            project,
            spaceModel,
            queryRunner,
            dto.subspaceModels,
          );
      }
      await queryRunner.commitTransaction();

      await this.commandBus.execute(
        new PropogateUpdateSpaceModelCommand({
          spaceModel: spaceModel,
          subspaceUpdates: modifiedSubspaces,
          spaces: spaces,
        }),
      );

      if (modifiedProductAllocations?.length > 0) {
        await this.commandBus.execute(
          new PropogateUpdateSpaceModelProductAllocationCommand({
            spaceModel: spaceModel,
            updatedAllocations: modifiedProductAllocations,
            spaces: spaces,
          }),
        );
      }

      return new SuccessResponseDto({
        message: 'SpaceModel updated successfully',
      });
    } catch (error) {
      await queryRunner.rollbackTransaction();
      if (error instanceof HttpException) {
        throw error;
      }

      throw new HttpException(
        error.message || 'Failed to update SpaceModel',
        HttpStatus.INTERNAL_SERVER_ERROR,
      );
    } finally {
      await queryRunner.release();
    }
  }

  async deleteSpaceModel(param: SpaceModelParam): Promise<BaseResponseDto> {
    const queryRunner = this.dataSource.createQueryRunner();
    await queryRunner.connect();
    await queryRunner.startTransaction();

    try {
      await this.validateProject(param.projectUuid);
      const spaceModel = await this.validateSpaceModel(
        param.spaceModelUuid,
        param.projectUuid,
      );

      if (spaceModel.subspaceModels?.length) {
        const deleteSubspaceUuids = spaceModel.subspaceModels.map(
          (subspace) => subspace.uuid,
        );

        await this.subSpaceModelService.clearModels(
          deleteSubspaceUuids,
          queryRunner,
        );
      }

      if (spaceModel.productAllocations?.length) {
        await this.spaceModelProductAllocationService.clearAllAllocations(
          spaceModel.uuid,
          queryRunner,
        );
      }

      await queryRunner.manager.update(
        this.spaceModelRepository.target,
        { uuid: param.spaceModelUuid },
        { disabled: true },
      );
      await queryRunner.commitTransaction();

      await this.commandBus.execute(
        new PropogateDeleteSpaceModelCommand({
          spaceModel: spaceModel,
          queryRunner,
        }),
      );

      return new SuccessResponseDto({
        message: `SpaceModel with UUID ${param.spaceModelUuid} deleted successfully.`,
        statusCode: HttpStatus.OK,
      });
    } catch (error) {
      await queryRunner.rollbackTransaction();

      const errorMessage =
        error instanceof HttpException
          ? error.message
          : 'An unexpected error occurred while deleting the SpaceModel';
      const statusCode =
        error instanceof HttpException
          ? error.getStatus()
          : HttpStatus.INTERNAL_SERVER_ERROR;

      throw new HttpException(errorMessage, statusCode);
    } finally {
      await queryRunner.release();
    }
  }

  async fetchModelSpaces(
    spaceModel: SpaceModelEntity,
    queryRunner?: QueryRunner,
  ) {
    const spaces = await (queryRunner
      ? queryRunner.manager.find(SpaceEntity, {
          where: {
            spaceModel: {
              uuid: spaceModel.uuid,
            },
            disabled: false,
          },
        })
      : this.spaceRepository.find({
          where: {
            spaceModel: {
              uuid: spaceModel.uuid,
            },
            disabled: false,
          },
        }));
    return spaces;
  }

  async linkSpaceModel(
    params: SpaceModelParam,
    dto: LinkSpacesToModelDto,
  ): Promise<BaseResponseDto> {
    const project = await this.validateProject(params.projectUuid);

    try {
      const spaceModel = await this.spaceModelRepository.findOne({
        where: { uuid: params.spaceModelUuid },
        relations: [
          'productAllocations',
          'subspaceModels',
          'productAllocations.product',
          'productAllocations.tag',
          'subspaceModels.productAllocations',
          'subspaceModels.productAllocations.product',
          'subspaceModels.productAllocations.tag',
        ],
      });

      if (!spaceModel) {
        throw new HttpException(
          `Space Model with UUID ${params.spaceModelUuid} not found`,
          HttpStatus.NOT_FOUND,
        );
      }

      const spaces = await this.spaceRepository.find({
        where: { uuid: In(dto.spaceUuids), disabled: false },
        relations: [
          'spaceModel',
          'devices',
          'subspaces',
          'productAllocations',
          'productAllocations.product',
          'productAllocations.tag',
          'subspaces.productAllocations',
          'subspaces.productAllocations.product',
          'subspaces.productAllocations.product.tag',
          'community',
        ],
      });

      if (!spaces.length) {
        throw new HttpException(
          `No spaces found for the given UUIDs`,
          HttpStatus.NOT_FOUND,
        );
      }

      await Promise.all(
        spaces.map(async (space) => {
          const hasDependencies =
            space.devices.length > 0 ||
            space.subspaces.length > 0 ||
            space.productAllocations.length > 0;

          if (!hasDependencies && !space.spaceModel) {
            await this.linkToSpace(space, spaceModel);
          } else if (dto.overwrite) {
            await this.removeSpaceOldSubspacesAndAllocations(space, project);
            await this.linkToSpace(space, spaceModel);
          }
        }),
      );

      return new SuccessResponseDto({
        message: 'Spaces linked successfully',
        data: dto.spaceUuids,
      });
    } catch (error) {
      throw new HttpException(
        `Failed to link space model: ${error.message}`,
        HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }

  async linkToSpace(
    space: SpaceEntity,
    spaceModel: SpaceModelEntity,
    queryRunner?: QueryRunner, // Make queryRunner optional
  ): Promise<void> {
    try {
      if (queryRunner) {
        await queryRunner.manager.update(SpaceEntity, space.uuid, {
          spaceModel,
        });
      } else {
        await this.spaceRepository.update(space.uuid, { spaceModel });
      }
      const spaceProductAllocations = spaceModel.productAllocations.map(
        (modelAllocation) =>
          this.spaceProductAllocationRepository.create({
            space,
            inheritedFromModel: modelAllocation,
            product: modelAllocation.product,
            tag: modelAllocation.tag,
          }),
      );
      if (queryRunner) {
        await queryRunner.manager.save(
          SpaceProductAllocationEntity,
          spaceProductAllocations,
        );
      } else {
        await this.spaceProductAllocationRepository.save(
          spaceProductAllocations,
        );
      }
      await Promise.all(
        spaceModel.subspaceModels.map(async (subspaceModel) => {
          const subspace = this.subspaceRepository.create({
            subspaceName: subspaceModel.subspaceName,
            subSpaceModel: subspaceModel,
            space: space,
          });
          if (queryRunner) {
            await queryRunner.manager.save(SubspaceEntity, subspace);
          } else {
            await this.subspaceRepository.save(subspace);
          }
          const subspaceAllocations = subspaceModel.productAllocations.map(
            (modelAllocation) =>
              this.subspaceProductAllocationRepository.create({
                subspace,
                inheritedFromModel: modelAllocation,
                product: modelAllocation.product,
                tag: modelAllocation.tag,
              }),
          );
          if (subspaceAllocations.length) {
            if (queryRunner) {
              await queryRunner.manager.save(
                SubspaceProductAllocationEntity,
                subspaceAllocations,
              );
            } else {
              await this.subspaceProductAllocationRepository.save(
                subspaceAllocations,
              );
            }
          }
        }),
      );
    } catch (error) {
      throw new HttpException(
        `Failed to link space ${space.uuid} to space model ${spaceModel.uuid}: ${error.message}`,
        HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }

  async removeSpaceOldSubspacesAndAllocations(
    space: SpaceEntity,
    project: ProjectEntity,
    queryRunner?: QueryRunner,
  ): Promise<void> {
    try {
      const spaceProductAllocationRepository = queryRunner
        ? queryRunner.manager.getRepository(SpaceProductAllocationEntity)
        : this.spaceProductAllocationRepository;

      const subspaceRepository = queryRunner
        ? queryRunner.manager.getRepository(SubspaceEntity)
        : this.subspaceRepository;

      const subspaceProductAllocationRepository = queryRunner
        ? queryRunner.manager.getRepository(SubspaceProductAllocationEntity)
        : this.subspaceProductAllocationRepository;

      if (space.productAllocations.length) {
        await spaceProductAllocationRepository.delete({
          uuid: In(
            space.productAllocations.map((allocation) => allocation.uuid),
          ),
        });
      }

      await Promise.all(
        space.subspaces.map(async (subspace) => {
          await subspaceRepository.update(
            { uuid: subspace.uuid },
            { disabled: true },
          );

          if (subspace.productAllocations.length) {
            await subspaceProductAllocationRepository.delete({
              uuid: In(
                subspace.productAllocations.map(
                  (allocation) => allocation.uuid,
                ),
              ),
            });
          }
        }),
      );

      if (space.devices.length > 0) {
        this.deviceService.addDevicesToOrphanSpace(space, project, queryRunner);
      }
    } catch (error) {
      throw new Error(
        `Failed to overwrite space ${space.uuid}: ${error.message}`,
      );
    }
  }

  async validateNameUsingQueryRunner(
    modelName: string,
    projectUuid: string,
    queryRunner: QueryRunner,
  ): Promise<void> {
    const isModelExist = await queryRunner.manager.findOne(SpaceModelEntity, {
      where: { modelName, project: { uuid: projectUuid }, disabled: false },
    });

    if (isModelExist) {
      throw new HttpException(
        `Model name ${modelName} already exists in the project with UUID ${projectUuid}.`,
        HttpStatus.CONFLICT,
      );
    }
  }

  async findOne(params: SpaceModelParam): Promise<BaseResponseDto> {
    try {
      await this.validateProject(params.projectUuid);
      const spaceModel = await this.validateSpaceModel(
        params.spaceModelUuid,
        params.projectUuid,
      );
      const response = this.formatSpaceModelResponse(spaceModel);

      return new SuccessResponseDto({
        message: 'SpaceModel retrieved successfully',
        data: response,
      });
    } catch (error) {
      throw new HttpException(
        `Failed to retrieve space model: ${error.message}`,
        HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }

  async validateSpaceModel(
    uuid: string,
    projectUuid?: string,
  ): Promise<SpaceModelEntity> {
    const query = this.buildSpaceModelQuery(projectUuid);
    const result = await query
      .andWhere('spaceModel.uuid = :uuid', { uuid })
      .getOne();

    if (!result) {
      throw new HttpException('space model not found', HttpStatus.NOT_FOUND);
    }
    return result;
  }

  private validateUniqueTags(allTags: ProcessTagDto[]) {
    const tagUuidSet = new Set<string>();
    const tagNameProductSet = new Set<string>();

    for (const tag of allTags) {
      if (tag.uuid) {
        if (tagUuidSet.has(tag.uuid)) {
          throw new HttpException(
            `Duplicate tag UUID found: ${tag.uuid}`,
            HttpStatus.BAD_REQUEST,
          );
        }
        tagUuidSet.add(tag.uuid);
      } else {
        if (!tag.name || !tag.productUuid) {
          throw new HttpException(
            `Tag name and product should not be null.`,
            HttpStatus.BAD_REQUEST,
          );
        }
        const tagKey = `${tag.name}-${tag.productUuid}`;
        if (tagNameProductSet.has(tagKey)) {
          throw new HttpException(
            `Duplicate tag found with name "${tag.name}" and product "${tag.productUuid}".`,
            HttpStatus.BAD_REQUEST,
          );
        }
        tagNameProductSet.add(tagKey);
      }
    }
  }

  private buildSpaceModelQuery(
    projectUuid: string,
  ): SelectQueryBuilder<SpaceModelEntity> {
    return this.spaceModelRepository
      .createQueryBuilder('spaceModel')
      .leftJoinAndSelect(
        'spaceModel.subspaceModels',
        'subspaceModels',
        'subspaceModels.disabled = :subspaceDisabled',
        { subspaceDisabled: false },
      )
      .leftJoinAndSelect(
        'subspaceModels.productAllocations',
        'subspaceModelProductAllocations',
      )
      .leftJoinAndSelect(
        'subspaceModelProductAllocations.tag',
        'subspaceModelTag',
      )
      .leftJoinAndSelect(
        'subspaceModelProductAllocations.product',
        'subspaceModelProduct',
      )
      .leftJoinAndSelect('spaceModel.productAllocations', 'productAllocations')
      .leftJoinAndSelect('productAllocations.product', 'allocatedProduct')
      .leftJoinAndSelect('productAllocations.tag', 'spaceModelTag')
      .leftJoinAndSelect(
        'productAllocations.product',
        'productAllocationsProduct',
      )
      .where('spaceModel.disabled = false')
      .andWhere('spaceModel.project = :projectUuid', { projectUuid });
  }

  private transformSpaceModelData(spaceModelsArray: SpaceModelEntity[]): any[] {
    if (!Array.isArray(spaceModelsArray)) return [];

    return spaceModelsArray.map((spaceModel) => ({
      uuid: spaceModel.uuid,
      createdAt: spaceModel.createdAt,
      updatedAt: spaceModel.updatedAt,
      modelName: spaceModel.modelName,
      disabled: spaceModel.disabled,
      subspaceModels: (spaceModel.subspaceModels ?? []).map((subspace) => ({
        uuid: subspace.uuid,
        createdAt: subspace.createdAt,
        updatedAt: subspace.updatedAt,
        subspaceName: subspace.subspaceName,
        disabled: subspace.disabled,
        productAllocations: subspace.productAllocations,
      })),
      productAllocations: spaceModel.productAllocations,
    }));
  }

  private formatSpaceModelResponse(spaceModel: SpaceModelEntity): any {
    return {
      uuid: spaceModel.uuid,
      createdAt: spaceModel.createdAt,
      updatedAt: spaceModel.updatedAt,
      modelName: spaceModel.modelName,
      disabled: spaceModel.disabled,
      subspaceModels:
        spaceModel.subspaceModels?.map((subspace) => ({
          uuid: subspace.uuid,
          createdAt: subspace.createdAt,
          updatedAt: subspace.updatedAt,
          subspaceName: subspace.subspaceName,
          disabled: subspace.disabled,
          productAllocations: subspace.productAllocations,
        })) ?? [],
      productAllocations: spaceModel.productAllocations,
    };
  }
}
